const treeData = {
  name: 'EV stock',
  type: '#FF0000', // Red
  level: '#8B0000', // Dark Red
  children: [
    {
      name: 'China',
      type: '#8B0000', // Dark red
      level: '#B8860B', // Dark Gold
      children: [
        {
          name: 'Buses',
          type: '#4B0082', // Indigo
          level: '#310062', // Dark Indigo
          children: [
            { name: 'BEV', value: 280, type: '#008000' }, // Green
            { name: 'PHEV', value: 300, type: '#008000' }, // Green
          ],
        },
        {
          name: 'Cars',
          type: '#4B0082', // Indigo
          level: '#310062', // Dark Indigo
          children: [
            { name: 'BEV', value: 930, type: '#008000' }, // Green
            { name: 'PHEV', value: 1280, type: '#008000' }, // Green
          ],
        },
      ],
    },
    {
      name: 'India',
      type: '#8B0000', // dark red
      level: '#B8860B', // Dark Gold
      children: [
        {
          name: 'Buses',
          type: '#4B0082', // Indigo
          level: '#310062', // Dark Indigo
          children: [
            { name: 'BEV', value: 100, type: '#008000' }, // Green
          ],
        },
        {
          name: 'Cars',
          type: '#4B0082', // Indigo
          level: '#310062', // Dark Indigo
          children: [
            { name: 'BEV', value: 70, type: '#008000' }, // Green
          ],
        },
      ],
    },
    {
      name: 'United Kingdom',
      type: '#8B0000', // dark red
      level: '#B8860B', // Dark Gold
      children: [
        {
          name: 'Cars',
          type: '#4B0082', // Indigo
          level: '#310062', // Dark Indigo
          children: [
            { name: 'BEV', value: 420, type: '#008000' }, // Green
            { name: 'PHEV', value: 870, type: '#008000' }, // Green
          ],
        },
        {
          name: 'Buses',
          type: '#4B0082', // Indigo
          level: '#310062', // Dark Indigo
          children: [
            { name: 'BEV', value: 300, type: '#008000' }, // Green
            { name: 'PHEV', value: 150, type: '#008000' }, // Green
          ],
        },
      ],
    },
    {
      name: 'USA',
      type: '#8B0000', // Dark red
      level: '#B8860B', // Dark Gold
      children: [
        {
          name: 'Cars',
          type: '#4B0082', // Indigo
          level: '#310062', // Dark Indigo
          children: [
            { name: 'BEV', value: 1200, type: '#008000' }, // Green
            { name: 'PHEV', value: 2000, type: '#008000' }, // Green
          ],
        },
      ],
    },
  ],
};

// Here Im setting the margin values to spread my tree structure on page
const margin = {
    top: 30,
    right: 130,
    bottom: 30,
    left: 90,
  },
  // assigning the height and width values which will be used to by SVG object
  width = 800 - margin.left - margin.right,
  height = 500 - margin.top - margin.bottom;

// Here Im selecting the div in the body page and appending the 'svg' element
//The width and height of svg element are set
// grouping the grouping element 'g' to svg to control overall style and transformations
const svg = d3
  .select('div')
  .append('svg')
  .attr('width', width + margin.right + margin.left)
  .attr('height', height + margin.top + margin.bottom)
  .append('g')
  .attr(
    'transform',
    'translate(' + margin.left + ',' + margin.top + ')',
  );

// Assigning the tree structure to 'root' using d3.hierarchy()
//Setting up the tree layout in the 'treelayout' by calling d3.tree() and setting size.
const root = d3.hierarchy(treeData);
const treeLayout = d3.tree().size([height, width]);
//paasing the hierarchy into the tree layout
treeLayout(root);

// Adding links between nodes as straight lines
// Sellecting all links with selectAll()
//Binding the root data to selected links with data(root.links)
//After entering in the data binded, it is grouped under the class 'links'
//Setting the stroke and style for the links using JSON data colors
//Drawing the curve for as the link between the nodes (circles)
svg
  .selectAll('.link')
  .data(root.links())
  .enter()
  .append('path')
  .attr('class', 'link')
  .style('stroke', (d) => d.target.data.type)
  .style('fill', 'none')
  .attr('d', function (d) {
    // Start point
    var sx = d.source.y,
      sy = d.source.x;

    // Control points
    var cx1 = (d.source.y + d.target.y) / 2,
      cy1 = d.source.x,
      cx2 = (d.source.y + d.target.y) / 2,
      cy2 = d.target.x;

    // End point
    var tx = d.target.y,
      ty = d.target.x;

    // Returning a path that goes to the left to right
    // curving through the two control points and ends at the target
    return `M${sx},${sy} C${cx1},${cy1} ${cx2},${cy2} ${tx},${ty}`;
  });

// Declaring a group for the nodes and binding the data of tree to it
//Entering the selection and setting the group based on child is internal/leaf
//setting the positions for the nodes
const node = svg
  .selectAll('.node')
  .data(root.descendants())
  .enter()
  .append('g')
  .attr(
    'class',
    (d) =>
      'node' +
      (d.children ? ' node--internal' : ' node--leaf'),
  )
  .attr('transform', (d) => `translate(${d.y},${d.x})`);

// setting circle as the node shape
//dynamically setting the node size for the nodes that have value in JSON
//Only leaf nodes have values defined in tree data
//Used math square root function and a 1.5 times division for leaf
// default size of 10 for other nodes
//Adding colors to the nodes
node
  .append('circle')
  .attr(
    'r',
    (d) =>
      d.data.value ? Math.sqrt(d.data.value) / 1.5 : 10, // dynamic radius setting
  )
  .style('stroke', (d) => d.data.type)
  .style('fill', (d) =>
    d.data.level ? d.data.level : 'none',
  );

// Adding text labels to the nodes, including values for leaf nodes
node
  .append('text')
  .attr('dy', '.45em')
  .attr('x', (d) => (d.children ? -15 : 28)) // x position based on leaf or not
  .attr('y', (d) => (d.children ? -20 : 0)) // y position based on leaf or not
  //using text anchor to set the label at the end or start
  .style('text-anchor', (d) =>
    d.children ? 'end' : 'start',
  )
  .text((d) => {
    // Appending value to the name if it's a leaf node
    return d.data.value
      ? `${d.data.name} (${d.data.value})`
      : d.data.name;
  });
